﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Telerik.Windows.Documents.RichTextBoxCommands;
using Telerik.Windows.Controls;

namespace AM.Common
{
	/// <summary>
	/// Redo derivative of the Telerik <c>MultipleUndoCommand</c>.
	/// </summary>
	public class MultiRedoCommand : MultipleUndoCommand
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="MultiRedoCommand"/> class.
		/// </summary>
		/// <param name="editor">The <c>RadRichTextBox</c>.</param>
		public MultiRedoCommand(RadRichTextBox editor)
			: base(editor)
		{
		}

		/// <summary>
		/// Command Execute override implementing Redo.
		/// </summary>
		/// <param name="parameter"><c>Int32</c> count of Redo actions to perform.</param>
		protected override void ExecuteOverride(object parameter)
		{
			int redoCount = Convert.ToInt32(parameter);
			for (int i = 0; i <= redoCount; i++)
			{
				base.AssociatedRichTextBox.Redo();
			}
		}

	}

	/// <summary>
	/// Undo derivative of the Telerik <c>MultipleUndoCommand</c>.
	/// </summary>
	public class MultiUndoCommand : MultipleUndoCommand
	{
		/// <summary>
		/// Initializes a new instance of the <see cref="MultiUndoCommand"/> class.
		/// </summary>
		/// <param name="editor">The editor.</param>
		public MultiUndoCommand(RadRichTextBox editor)
			: base(editor)
		{
		}

		/// <summary>
		/// Command Execute override implementing Undo.
		/// </summary>
		/// <param name="parameter"><c>Int32</c> count of Undo actions to perform.</param>
		protected override void ExecuteOverride(object parameter)
		{
			int undoCount = Convert.ToInt32(parameter);
			for (int i = 0; i <= undoCount; i++)
			{
				base.AssociatedRichTextBox.Undo();
			}
		}

	}

}
