﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Documents.History;
using Telerik.Windows.Documents.Model;
using Telerik.Windows.Documents.RichTextBoxCommands;
using Telerik.Windows.Documents.RichTextBoxCommands.CommandEventArgs;

namespace AM.Common.UI
{
	/// <summary>
	/// Handles the attachment of the MultiRedoCommand to a <c>RadRichTextBox</c>.
	/// </summary>
	public class RichTextBoxRedoRibbonUI
	{
		private readonly static DependencyProperty CommandStateChangedHandlerProperty;

		private readonly static DependencyProperty ControlStateChangedHandlerProperty;

		private readonly static DependencyProperty ControlStateChangedInternallyHandlerProperty;

		/// <summary>
		/// The Redo command property.
		/// </summary>
		public readonly static DependencyProperty RichTextCommandProperty;

		static RichTextBoxRedoRibbonUI()
		{
			RichTextBoxRedoRibbonUI.CommandStateChangedHandlerProperty = DependencyProperty.RegisterAttached("CommandStateChangedHandler", typeof(object), typeof(RichTextBoxRedoRibbonUI), null);
			RichTextBoxRedoRibbonUI.ControlStateChangedInternallyHandlerProperty = DependencyProperty.RegisterAttached("ControlStateChangedInternallyHandler", typeof(object), typeof(RichTextBoxRedoRibbonUI), null);
			RichTextBoxRedoRibbonUI.ControlStateChangedHandlerProperty = DependencyProperty.RegisterAttached("ControlStateChangedHandler", typeof(object), typeof(RichTextBoxRedoRibbonUI), null);
			RichTextBoxRedoRibbonUI.RichTextCommandProperty = DependencyProperty.RegisterAttached("RichTextCommand", typeof(RichTextBoxCommandBase), typeof(RichTextBoxRedoRibbonUI), new PropertyMetadata(RichTextBoxRedoRibbonUI.OnRichTextBoxCommandChanged));
		}

		private static void ClearAttachedProperties(DependencyObject obj)
		{
			obj.ClearValue(RichTextBoxRedoRibbonUI.ControlStateChangedHandlerProperty);
			obj.ClearValue(RichTextBoxRedoRibbonUI.CommandStateChangedHandlerProperty);
			obj.ClearValue(RichTextBoxRedoRibbonUI.ControlStateChangedInternallyHandlerProperty);
		}



		/// <summary>
		/// Initializes a new instance of the <see cref="RichTextBoxRedoRibbonUI"/> class.
		/// </summary>
		public RichTextBoxRedoRibbonUI()
		{
		}


		private static void SetCommandStateChangedHandler(DependencyObject obj, object value)
		{
			obj.SetValue(RichTextBoxRedoRibbonUI.CommandStateChangedHandlerProperty, value);
		}

		private static object GetCommandStateChangedHandler(DependencyObject obj)
		{
			return obj.GetValue(RichTextBoxRedoRibbonUI.CommandStateChangedHandlerProperty);
		}

		private static void SetControlStateChangedHandler(DependencyObject obj, object value)
		{
			obj.SetValue(RichTextBoxRedoRibbonUI.ControlStateChangedHandlerProperty, value);
		}

		private static object GetControlStateChangedHandler(DependencyObject obj)
		{
			return obj.GetValue(RichTextBoxRedoRibbonUI.ControlStateChangedHandlerProperty);
		}

		/// <summary>
		/// Gets the redo command.
		/// </summary>
		/// <param name="obj">The DependencyObject.</param>
		/// <returns>RichTextBoxCommandBase</returns>
		public static RichTextBoxCommandBase GetRichTextCommand(DependencyObject obj)
		{
			return (RichTextBoxCommandBase)obj.GetValue(RichTextBoxRedoRibbonUI.RichTextCommandProperty);
		}

		/// <summary>
		/// Sets the redo command.
		/// </summary>
		/// <param name="obj">The DependencyObject.</param>
		/// <param name="value">RichTextBoxCommandBase value.</param>
		public static void SetRichTextCommand(DependencyObject obj, RichTextBoxCommandBase value)
		{
			obj.SetValue(RichTextBoxRedoRibbonUI.RichTextCommandProperty, value);
		}

		private static void AttachMultipleRedoCommand(MultipleRedoControl multipleRedoControl, MultiRedoCommand oldCommand, MultiRedoCommand newCommand)
		{
			EventHandler eventHandler = null;
			EventHandler<MultipleUndoExecutedEventArgs> eventHandler1 = null;
			if (oldCommand != null)
			{
				EventHandler oldHandler = RichTextBoxRedoRibbonUI.GetCommandStateChangedHandler(multipleRedoControl) as EventHandler;
				if (oldHandler != null)
				{
					oldCommand.DocumentHistoryChanged -= oldHandler;
				}
				EventHandler<MultipleUndoExecutedEventArgs> oldmultipleUndoHandler = RichTextBoxRedoRibbonUI.GetControlStateChangedHandler(multipleRedoControl) as EventHandler<MultipleUndoExecutedEventArgs>;
				if (oldmultipleUndoHandler != null)
				{
					multipleRedoControl.MultipleUndoTriggered -= oldmultipleUndoHandler;
				}
			}
			if (newCommand != null)
			{
				if (eventHandler == null)
				{
					eventHandler = (object s, EventArgs e) =>
					{
						Type historyType = newCommand.AssociatedRichTextBox.Document.History.GetType();

						Stack<HistoryAction> redoStack;
						redoStack = historyType.InvokeMember("redoStack", BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.GetField, null,
							newCommand.AssociatedRichTextBox.Document.History, null) as Stack<HistoryAction>;
						
						var mupList = new List<MultipleUndoPoint>();
						MultipleUndoPoint mup = null;
						
						bool inTyping = false;
						int inx = 0;
						
						foreach (HistoryCommandAction a in redoStack)
						{
							if (((inTyping) && (a.Command.Name.CompareTo("Typing") != 0)))
							{
								mup.DisplayText = "Typing \"" + mup.DisplayText;
								if (mup.DisplayText.Length > 49)
								{
									mup.DisplayText = mup.DisplayText.Substring(0, 47) + "...";
								}
								else
								{
									mup.DisplayText += "\"";
								}
								mup.UndoStackIndex = inx;
								mupList.Add(mup);
								inTyping = false;
							}

							if ((!inTyping) && (a.Command.Name.CompareTo("Typing") == 0))
							{
								inTyping = true;
								mup = new MultipleUndoPoint();
								//mup.DisplayText = a.Command.Name + " \"";
							}

							if (inTyping)
							{
								var ctx = a.CommandContext;
								if (ctx != null)
								{
									try
									{
										Type commandContextType = ctx.GetType();
										String text = commandContextType.InvokeMember("text", BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.GetField, null,
											ctx, null) as String;

										if (!String.IsNullOrEmpty(text))
										{
											mup.DisplayText = text + mup.DisplayText;
										}
									}
									catch /*(System.Exception ex)*/
									{
										// Debug.WriteLine(ex.Message);
									}

								}
							}
							else
							{
								mup = new MultipleUndoPoint();
								mup.DisplayText = a.Command.Name;
								mup.UndoStackIndex = inx;

								mupList.Add(mup);
							}

							if (mupList.Count >= 20)
							{
								break;
							}
							inx++;
						}

						if (inTyping)
						{
							mup.DisplayText = "Typing \"" + mup.DisplayText;
							if (mup.DisplayText.Length > 49)
							{
								mup.DisplayText = mup.DisplayText.Substring(0, 47) + "...";
							}
							else
							{
								mup.DisplayText += "\"";
							}
							mup.UndoStackIndex = inx;
							mupList.Add(mup);
							inTyping = false;
						}

						if (!multipleRedoControl.IsTemplateApplied)
						{
							multipleRedoControl.ItemsCollection = mupList;
							return;
						}
						else
						{
							multipleRedoControl.SetItemsControlSource(mupList);
							return;
						}
					}
					;
				}
				EventHandler newHandler = eventHandler;
				RichTextBoxRedoRibbonUI.SetCommandStateChangedHandler(multipleRedoControl, newHandler);
				newCommand.DocumentHistoryChanged += newHandler;
				newHandler(newCommand, EventArgs.Empty);
				if (eventHandler1 == null)
				{
					eventHandler1 = (object s, MultipleUndoExecutedEventArgs e) => newCommand.Execute(e.UndoStackIndex);
				}
				EventHandler<MultipleUndoExecutedEventArgs> multipleUndoHandler = eventHandler1;
				multipleRedoControl.MultipleUndoTriggered += multipleUndoHandler;
				RichTextBoxRedoRibbonUI.SetControlStateChangedHandler(multipleRedoControl, multipleUndoHandler);
			}
		}

		private static void OnRichTextBoxCommandChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
		{
			if (o as MultipleRedoControl != null && (e.NewValue as MultiRedoCommand != null || e.OldValue as MultiRedoCommand != null))
			{
				RichTextBoxRedoRibbonUI.AttachMultipleRedoCommand((MultipleRedoControl)o, e.OldValue as MultiRedoCommand, e.NewValue as MultiRedoCommand);
			}
			if (e.OldValue != null && e.NewValue == null)
			{
				RichTextBoxRedoRibbonUI.ClearAttachedProperties(o);
			}
		}

	}

}
